/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package a0b36pr2_du2.rychly_balik.model.db_service;

import a0b36pr2_du2.rychly_balik.model.RychlyBalikException;
import a0b36pr2_du2.rychly_balik.model.Zasilka;
import a0b36pr2_du2.rychly_balik.model.doprava.Dodavka;
import a0b36pr2_du2.rychly_balik.model.doprava.Doprava;
import a0b36pr2_du2.rychly_balik.model.doprava.Kolo;
import a0b36pr2_du2.rychly_balik.model.doprava.Mhd;
import a0b36pr2_du2.rychly_balik.model.doprava.Pickup;
import a0b36pr2_du2.rychly_balik.model.doprava.Riditelny;
import a0b36pr2_du2.rychly_balik.model.doprava.Riditelny.Opravneni;
import a0b36pr2_du2.rychly_balik.model.personalistika.AdministrativniPracovnik;
import a0b36pr2_du2.rychly_balik.model.personalistika.Pracovnik;
import a0b36pr2_du2.rychly_balik.model.personalistika.VykonnyPracovnik;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author hapallen
 */
public class DbService {

    private String url = "jdbc:derby:hapallenDB;create=true";
    private String user = "hapallen";
    private String password = "hapalen";
    private Connection connection = null;
    private Statement statement = null;

    public Connection getConnection() throws InstantiationException, IllegalAccessException, ClassNotFoundException, SQLException {
        if (connection == null) {
            Class.forName("org.apache.derby.jdbc.EmbeddedDriver").newInstance();
            connection =
                    DriverManager.getConnection(url, user, password);
        }
        return connection;
    }

    public Statement getStatement() throws RychlyBalikException {
        try {
            if (statement == null) {
                this.statement = getConnection().createStatement();
            }
        } catch (Exception e) {
            throw new RychlyBalikException("Pri pokusu o dotaz na DB doslo k chybe.");
        }
        return statement;
    }

    public void closeConnection() throws RychlyBalikException {
        try {
            if (this.connection != null) {
                if (statement != null) {
                    // uzavření dotazu i všech výsledků
                    statement.close();
                }
                // uzavření spojení
                connection.close();
            }
        } catch (Exception e) {
            throw new RychlyBalikException("Pri pokusu o dotaz na DB doslo k chybe.");
        }
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getUrl() {
        return url;
    }

    public void setUrl(String url) {
        this.url = url;
    }

    public String getUser() {
        return user;
    }

    public void setUser(String user) {
        this.user = user;
    }

    public void createTables() throws SQLException {
        try {
            int i =
                    getStatement().
                    executeUpdate("CREATE TABLE APP.DOPRAVNI_PROSTREDKY (    DP_ID INTEGER not null GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1) primary key,"
                    + "DP_TYP  VARCHAR(15),    DP_NOSNOST  DECIMAL(5),   DP_RYCHLOST  DECIMAL(5),    OP_ID INTEGER) ");
            i =
                    getStatement().
                    executeUpdate("CREATE INDEX DOPRAVNI_PROSTREDKY_TYP_IDX ON DOPRAVNI_PROSTREDKY (DP_TYP)");
            i =
                    getStatement().
                    executeUpdate("CREATE UNIQUE INDEX SQL10013011571454 ON DOPRAVNI_PROSTREDKY (DP_ID) ");

            i =
                    getStatement().executeUpdate("INSERT INTO APP.DOPRAVNI_PROSTREDKY (DP_TYP,DP_NOSNOST,DP_RYCHLOST) VALUES( 'DODAVKA', 400, 30)");
        } catch (RychlyBalikException ex) {
            Logger.getLogger(DbService.class.getName()).log(Level.SEVERE, null, ex);
        }

    }

    /**
     * nacte z DB dopravni prostredky
     * @return seznam dopravnich prostredku
     * @throws RychlyBalikException
     */
    public List<Doprava> loadDopravniProstredky() throws RychlyBalikException {
        List<Doprava> dopravniProstredky = new LinkedList<Doprava>();
        try {
            Doprava doprava = null;
            ResultSet rs =
                    getStatement().executeQuery("SELECT  * from APP.DOPRAVNI_PROSTREDKY");
            while (rs.next()) {
                Integer id = rs.getInt("DP_ID");
                String typ = rs.getString("DP_TYP");
                double nosnost = rs.getDouble("DP_NOSNOST");
                double rychlost = rs.getDouble("DP_RYCHLOST");

                // vytvorim objekt a pridam do seznamu
                Doprava.TypDopravy typDopravy = Doprava.TypDopravy.valueOf(typ);
                switch (typDopravy) {
                    case KOLO:
                        doprava = new Kolo(nosnost, rychlost);
                        break;
                    case DODAVKA:
                        doprava = new Dodavka();
                        break;
                    case MHD:
                        doprava = new Mhd(nosnost);
                        break;
                    case PICK_UP:
                        doprava = new Pickup(nosnost);
                        break;
                    default:
                        throw new RychlyBalikException("Jiny dopravni prostredek neni mozny.");
                }
                doprava.setId(id);
                dopravniProstredky.add(doprava);
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
            throw new RychlyBalikException("Chyba pri ziskavani dopravniho prostredku: " + e.
                    getMessage());
        }
        return dopravniProstredky;
    }

    /**
     * nacte z DB vsechny pracovniky
     * @return seznam pracovniku v DB
     * @throws RychlyBalikException
     */
    public List<Pracovnik> loadPracovnici() throws RychlyBalikException {
        List<Pracovnik> pracovnici = new LinkedList<Pracovnik>();
        try {
            Pracovnik pracovnik = null;
            ResultSet rs =
                    getStatement().executeQuery("SELECT  * from APP.PRACOVNICI");
            while (rs.next()) {
                int id = rs.getInt("P_ID");
                String jmeno = rs.getString("P_JMENO");
                String prijmeni = rs.getString("P_PRIJMENI");
                String typ = rs.getString("P_TYP");
                // vytvorim objekt a pridam do seznamu
                Pracovnik.TypPracovnika typPracovnika = Pracovnik.TypPracovnika.
                        valueOf(typ);

                switch (typPracovnika) {
                    case ADM:
                        pracovnik = new AdministrativniPracovnik(id);
                        break;
                    case EXEC:
                        pracovnik = new VykonnyPracovnik(id);
                        break;
                    default:
                        throw new RychlyBalikException("Jiny pracovnik neni mozny.");
                }
                pracovnik.setJmeno(jmeno);
                pracovnik.setPrijmeni(prijmeni);
                pracovnik.setId(id);
                if (pracovnik instanceof VykonnyPracovnik) {
                    VykonnyPracovnik vp = (VykonnyPracovnik) pracovnik;
                    //nastav jeho opravneni
                    vp.setOpravneni(getOpravneniProPracovnika(pracovnik));
                }
                pracovnici.add(pracovnik);
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
            throw new RychlyBalikException("Chyba pri ziskavani pracovnika: " + e.
                    getMessage());
        }
        return pracovnici;
    }

    /**
     * nacte z DB vsechny  zasilky
     * @return seznam zasilek v DB
     * @throws RychlyBalikException
     */
    public List<Zasilka> loadZasilky() throws RychlyBalikException {
        List<Zasilka> zasilky = new LinkedList<Zasilka>();
        try {
            Zasilka zasilka = null;
            ResultSet rs =
                    getStatement().executeQuery("SELECT Z_ID, Z_HMOTNOST, Z_VZDALENOST, P_ID from APP.ZASILKY");
            while (rs.next()) {
                int id = rs.getInt("Z_ID");
                double hmotnost = rs.getDouble("Z_HMOTNOST");
                double vzdalenost = rs.getDouble("Z_VZDALENOST");
                int pracovnikId = rs.getInt("P_ID");

                // vytvorim objekt a pridam do seznamu
                Zasilka z =
                        new Zasilka(hmotnost, vzdalenost);
                z.setId(id);
                Pracovnik p = getPracovnikById(pracovnikId);
                zasilka.setPracovnik(p);
                zasilky.add(z);
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
            throw new RychlyBalikException("Chyba pri ziskavani zasilky: " + e.
                    getMessage());
        }
        return zasilky;
    }

    /**
     * vyhleda v DB pracovnika podle zadaneho ID
     * @param id
     * @return vyhledany pracovnik, null, pokud neexistuje
     * @throws InstantiationException
     * @throws SQLException
     * @throws IllegalAccessException
     * @throws ClassNotFoundException
     * @throws RychlyBalikException
     */
    public Pracovnik getPracovnikById(Integer id) throws InstantiationException, SQLException, IllegalAccessException, ClassNotFoundException, RychlyBalikException {
        Pracovnik pracovnik = null;
        PreparedStatement preparedStatement;
        String getPracovnikByIdString =
                "SELECT  * from APP.PRACOVNICI WHERE  P_ID = ?";
        preparedStatement =
                getConnection().prepareStatement(getPracovnikByIdString);

        preparedStatement.setInt(1, id);
        ResultSet rs = preparedStatement.executeQuery();
        while (rs.next()) {
            String jmeno = rs.getString("P_JMENO");
            String prijmeni = rs.getString("P_PRIJMENI");
            String typ = rs.getString("P_TYP");

            // vytvorim objekt a pridam do seznamu
            Pracovnik.TypPracovnika typPracovnika = Pracovnik.TypPracovnika.
                    valueOf(typ);
            switch (typPracovnika) {
                case ADM:
                    pracovnik =
                            new AdministrativniPracovnik(id);
                    break;
                case EXEC:
                    pracovnik =
                            new VykonnyPracovnik(id);
                    break;
                default:
                    throw new RychlyBalikException("Jiny pracovnik neni mozny.");
            }
            pracovnik.setJmeno(jmeno);
            pracovnik.setPrijmeni(prijmeni);
            if (pracovnik instanceof VykonnyPracovnik) {
                VykonnyPracovnik vp = (VykonnyPracovnik) pracovnik;
                //nastav jeho opravneni
                vp.setOpravneni(getOpravneniProPracovnika(pracovnik));
            }
        }
        return pracovnik;
    }

    /**
     * vyhleda v DB opravneni pro zadaneho pracovnika
     * @param pracovnikId
     * @return
     * @throws SQLException
     * @throws InstantiationException
     * @throws IllegalAccessException
     * @throws ClassNotFoundException
     */
    public Set<Opravneni> getOpravneniProPracovnika(Pracovnik pracovnik) throws SQLException, InstantiationException, IllegalAccessException, ClassNotFoundException {
        Set<Opravneni> opravneni = new HashSet<Opravneni>();
        PreparedStatement preparedStatement;
        String queryString =
                "SELECT  OP_TYP from APP.OPRAVNENI o INNER JOIN APP.OPRAVNENI_PRACOVNIKU op ON o.OP_ID = op.OP_ID WHERE  P_ID = ?";
        preparedStatement =
                getConnection().prepareStatement(queryString);

        preparedStatement.setInt(1, pracovnik.getId());
        ResultSet rs = preparedStatement.executeQuery();

        while (rs.next()) {
            String typ = rs.getString("OP_TYP");
            Opravneni op = Opravneni.valueOf(typ);
            opravneni.add(op);
        }
        return opravneni;
    }

    /*
    public Zasilka getZasilkaById(Integer id) {
    Zasilka zasilka = null;
    return zasilka;
    }

    public Doprava getDopravaById(Integer id) {
    Doprava doprava = null;
    return doprava;
    }
     * 
     */
    /**
     * smaze zadany dopravni prostredek z DB
     * @param dopravni prostredek ke smazani
     * @throws RychlyBalikException
     */
    public void deleteDopravniProstredek(Doprava doprava) throws RychlyBalikException {
        if (doprava.getId() == null) {
            //dopravni prostredek nema id --> neni vubec v DB, takze jej nemusim odebirat
            return;
        }
        try {
            PreparedStatement preparedStatement;
            String queryString =
                    "DELETE FROM APP.DOPRAVNI_PROSTREDKY WHERE DP_ID = ?";
            preparedStatement =
                    getConnection().prepareStatement(queryString);
            preparedStatement.setInt(1, doprava.getId());
           int i = preparedStatement.executeUpdate();

        } catch (Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
            throw new RychlyBalikException("Chyba pri mazani dopravniho prostredku: " + e.
                    getMessage());
        }
    }

    /**
     * smaze zasilku z DB
     * @paramzasilka ke smazani
     * @throws RychlyBalikException
     */
    public void deleteZasilka(Zasilka zasilka) throws RychlyBalikException {
        if (zasilka.getId() == null) {
            //zasilka nema id --> neni vubec v DB, takze jej nemusim odebirat
            return;
        }
        try {
            PreparedStatement preparedStatement;
            String queryString =
                    "DELETE FROM APP.ZASILKY WHERE Z_ID = ?";
            preparedStatement =
                    getConnection().prepareStatement(queryString);
            preparedStatement.setInt(1, zasilka.getId());
            int i = preparedStatement.executeUpdate();

        } catch (Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
            throw new RychlyBalikException("Chyba pri mazani zasilky: " + e.
                    getMessage());
        }
    }

    /**
     * smaze pracovnika z DB
     * @param pracovnik ke smazani
     * @throws RychlyBalikException
     */
    public void deletePracovnik(Pracovnik pracovnik) throws RychlyBalikException {
        if (pracovnik.getId() == null) {
            //pracovnik nema id --> neni vubec v DB, takze jej nemusim odebirat
            return;
        }
        try {
            getStatement().execute("DELETE FROM APP.ZASILKY WHERE P_ID = " + pracovnik.
                    getId());
            getStatement().execute("DELETE FROM APP.OPRAVNENI_PRACOVNIKU WHERE P_ID = " + pracovnik.
                    getId());
            getStatement().execute("DELETE FROM APP.PRACOVNICI WHERE P_ID = " + pracovnik.
                    getId());
        } catch (Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
            throw new RychlyBalikException("Chyba pri mazani pracovnika: " + e.
                    getMessage());
        }
    }

    /**
     * vlozi pracovnika do DB a nastavi ji id podle DB
     * @param pracovnik
     * @return aktuallilzovany pracocvnik
     * @throws RychlyBalikException
     */
    public Pracovnik addPracovnik(Pracovnik pracovnik) throws RychlyBalikException {
        try {

            PreparedStatement preparedStatement;
            String queryString =
                    "INSERT INTO APP.PRACOVNICI (P_JMENO, P_PRIJMENI, P_TYP) VALUES (?,?,?)";
            preparedStatement =
                    getConnection().prepareStatement(queryString, Statement.RETURN_GENERATED_KEYS);
            preparedStatement.setString(1, pracovnik.getJmeno());
            preparedStatement.setString(2, pracovnik.getPrijmeni());
            preparedStatement.setString(3, pracovnik.getTypPracovnika().name());
            int i = preparedStatement.executeUpdate();

            ResultSet rs = preparedStatement.getGeneratedKeys();
            // nastavim pracovnikovi ID, ktere mu autoinkrementem pridelila DB
            while (rs.next()) {
                int id = rs.getInt(1);
                System.out.println("vlozene id: " + id);
                pracovnik.setId(id);
                break;
            }
            // nejprve musim vytvorit pracovnika a ted mohu nastavit jeho opravneni
            if (pracovnik instanceof VykonnyPracovnik) {
                VykonnyPracovnik vp = (VykonnyPracovnik) pracovnik;
                addOpravneniPracovnikovi(vp);
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
            throw new RychlyBalikException("Nastala chyba pri ukladani pracovnika do DB: " + e.
                    getMessage());
        }
        return pracovnik;
    }

    /**
     * ulozi opravneni zadaneho pracovnika do DB
     * @param vykonnyPracovnik
     * @throws SQLException
     * @throws InstantiationException
     * @throws IllegalAccessException
     * @throws ClassNotFoundException
     */
    public void addOpravneniPracovnikovi(VykonnyPracovnik vykonnyPracovnik) throws SQLException, InstantiationException, IllegalAccessException, ClassNotFoundException {
        PreparedStatement preparedStatement;
        String queryString =
                "INSERT INTO APP.OPRAVNENI_PRACOVNIKU (OP_ID,P_ID) VALUES (?,?)";
        preparedStatement =
                getConnection().prepareStatement(queryString);
        for (Opravneni op : vykonnyPracovnik.getOpravneni()) {
            preparedStatement.setInt(1, getIdOpravneni(op));
            preparedStatement.setInt(2, vykonnyPracovnik.getId());
            int i = preparedStatement.executeUpdate();
        }
    }

    /**
     * vlozi zasilku do DB a nastavi ji id podle DB
     * @param zasilka
     * @return aktualizovana zasilka
     * @throws RychlyBalikException
     */
    public Zasilka addZasilka(Zasilka zasilka) throws RychlyBalikException {
        try {
            PreparedStatement preparedStatement;
            String queryString =
                    "INSERT INTO APP.ZASILKY (Z_HMOTNOST, Z_VZDALENOST, P_ID) VALUES (?,?,?)";
            preparedStatement =
                    getConnection().prepareStatement(queryString, Statement.RETURN_GENERATED_KEYS);
            preparedStatement.setDouble(1, zasilka.getHmotnost());
            preparedStatement.setDouble(2, zasilka.getVzdalenost());
            if (zasilka.getPracovnik() != null) {
                preparedStatement.setInt(3, zasilka.getPracovnik().getId());
            }
            int i = preparedStatement.executeUpdate();
            ResultSet rs = preparedStatement.getGeneratedKeys();
            // nastavim pracovnikovi ID, ktere mu autoinkrementem pridelila DB
            while (rs.next()) {
                int id = rs.getInt(1);
                System.out.println("vlozene id: " + id);
                zasilka.setId(id);
                break;
            }

        } catch (Exception e) {
            System.err.println(e.getMessage());
            e.printStackTrace();
            throw new RychlyBalikException("Nastala chyba pri ukladani zasilky do DB: " + e.
                    getMessage());
        }
        return zasilka;
    }

    /**
     * vlozi dopravni prostredek do DB a nastavi mu id podle DB
     * @param dopravniProstredek
     * @return aktualizovany dopravni prostredek
     * @throws RychlyBalikException
     */
    public Doprava addDopravniProstredek(Doprava dopravniProstredek) throws RychlyBalikException {
        try {
            PreparedStatement preparedStatement;
            String queryString =
                    "INSERT INTO APP.DOPRAVNI_PROSTREDKY (DP_TYP, DP_NOSNOST, DP_RYCHLOST, OP_ID) VALUES (?,?,?,?)";
            preparedStatement =
                    getConnection().prepareStatement(queryString, Statement.RETURN_GENERATED_KEYS);
            preparedStatement.setString(1, dopravniProstredek.getTypDopravy().
                    name());
            preparedStatement.setDouble(2, dopravniProstredek.getNosnost());
            preparedStatement.setDouble(3, dopravniProstredek.getRychlost());

            if (dopravniProstredek instanceof Riditelny) {
                Riditelny dp = (Riditelny) dopravniProstredek;
                preparedStatement.setInt(4, getIdOpravneni(dp.getOpravneni()));
            }
            int i = preparedStatement.executeUpdate();

            ResultSet rs = preparedStatement.getGeneratedKeys();
            // nastavim pracovnikovi ID, ktere mu autoinkrementem pridelila DB
            while (rs.next()) {
                int id = rs.getInt(1);
                System.out.println("vlozene id: " + id);
                dopravniProstredek.setId(id);
                break;
            }
        } catch (Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
            throw new RychlyBalikException("Nastala chyba pri ukladani dopravniho prostredku do DB: " + e.
                    getMessage());
        }
        return dopravniProstredek;
    }

    /**
     * zjisti id daneho opravneni v DB
     * @param opravneni
     * @return id zdaneho opravneni
     */
    protected Integer getIdOpravneni(Opravneni opravneni) {
        try {
            PreparedStatement preparedStatement;
            String queryString =
                    "SELECT  OP_ID from APP.OPRAVNENI where OP_TYP = ?";
            preparedStatement =
                    getConnection().prepareStatement(queryString);
            preparedStatement.setString(1, opravneni.name());

            ResultSet rs = preparedStatement.executeQuery();
            while (rs.next()) {
                return rs.getInt("OP_ID");
            }
        } catch (Exception ex) {
            return 0;
        }
        return 0;
    }
}


